use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
API
CPAN
Chase
Clonable
File
GPL
HavingFilename
HavingGeneratedBy
IPC
Inc
Index
LGPL
MailRc
ModList
PackagesDetails
Param
Peter
Pixmaps
Proc
Readable
Role
Shangov
Venda
Whitener
Whois
Writable
authorid
capoeirab
chapterid
cpanid
github
ino
lib
pshangov
stopwords
wordlist
